import * as React from 'react';
import * as classNames from 'classnames';
import Icon from '../../../component/Icon';
import Text, { Props as TextProps } from '../../../component/Text';
import isSvgFile from '../../../utils/isSvgFile';

export type Icon = ((isActive: boolean) => string) | string;

export interface Props {
  /** 图标的地址 */
  icon?: Icon;
  /** 文本的内容 */
  text: string;
  /** 图标样式 */
  iconCls?: string;
  /** 文本样式 */
  textCls?: string;
  ellipsis?: TextProps['ellipsis'];
  active?: boolean;
}

const prefixCls = 'yj-component-icontext';

class IconText extends React.PureComponent<Props> {
  public render(): (JSX.Element | null)[] {
    const { icon, text, ellipsis, iconCls, textCls, active } = this.props;
    const iconStr =  typeof icon === 'function' ? icon(!!active) : icon;

    return [
      iconStr ? (
        isSvgFile(iconStr) ? (
          <Icon key="icon" className={classNames(prefixCls, iconCls)} src={iconStr} />
        ) : (
          <img key="icon" className={classNames(prefixCls, iconCls)} src={iconStr} />
        )
      ) : null,
      <Text
        key="text"
        className={textCls}
        style={{
          whiteSpace: 'nowrap'
        }}
        ellipsis={ellipsis}
      >{text}</Text>
    ];
  }
}

export default IconText;
