import Base, { ReadOnly } from '../../_base/datatype';
import Request from '../request';

interface Props {
  request?: Request;
}

type List<T> = Array<T>;

class ObListMap<Key, Value> extends Base<Map<Key, List<ReadOnly<Value>>>> {
  constructor(props?: Props) {
    super(props && props.request);
  }

  protected value: Map<Key, List<ReadOnly<Value>>> = new Map();

  public set(key: Key, value: List<ReadOnly<Value>>): void {
    this.value.set(this.stringify(key), value);

    if (this.request) {
      if (value.length > 0) {
        this.request.state.set('complete');
      } else {
        this.request.state.set('empty');
      }
    }
    this.updateDataId();
  }

  public get(key: Key): List<ReadOnly<Value>> | undefined {
    return this.value.get(this.stringify(key));
  }

  public forEach(callback: (dataitem: List<ReadOnly<Value>>, key: Key) => void): void {
    this.value.forEach(callback);
  }

  /** 清空所有的数据 */
  public clear(): void {
    this.value.clear();
  }

  public keys(): IterableIterator<Key> {
    return this.value.keys();
  }
}

export default ObListMap;