import * as React from 'react';
import * as classNames from 'classnames';
import Icon from '../../Icon';
import Container from '../../Container';
import svgRootPath from '../../../utils/svgRootPath';
import sheet, { getActionName } from '../../../model/behavior/sheet';
import PageStack from '../../PageStack';
import './index.scss';
import { ChildrenProps } from '../index';
const arrowSvg = svgRootPath + require('../../../assets/arrow.svg').id;

export interface Props extends ChildrenProps {
}

const prefixCls = 'yj-component-map-control';
class Return extends React.PureComponent<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

  public onClick(): void {
    const { map, service } = this.props;
    const stackComponent = PageStack.getStackComponent();

    if (map && service) {
      const mapPoint = service.center.mapPoint;

      if (mapPoint) {
        map.panTo(new BMap.Point(mapPoint.lng, mapPoint.lat));
      }
    }

    if (stackComponent) {
      stackComponent.behavior.trigger({
        actionName: getActionName(sheet.MapControl, 2, this.props),
      });
    }
  }

  public render(): JSX.Element {
    return (
      <Container
        className={`${prefixCls}-item`}
        alignItems="center"
        justifyContent="center"
        onClick={this.onClick}
      >
        <Icon
          className={classNames(`${prefixCls}-svg`, `${prefixCls}-orignMark`)}
          src={arrowSvg}
        />
      </Container>
    );
  }
}

export default Return;
