import * as clssNames from 'classnames';
import * as React from 'react';
import sheet, { DetailParams, getActionName } from '../../model/behavior/sheet';
import yj from '../../typings';
import Auth, { Props as AuthProps } from '../Auth';
import PageStack from '../PageStack';
import * as styles from './index.scss';

export interface Props extends yj.ComponentProps, yj.ComponentBehavior {
  children?: any;
  tel: string;
  onClick?: (e: React.MouseEvent<HTMLDivElement>) => void;
  subscribeContent?: AuthProps['subscribeContent'];
}

const ua = navigator.userAgent.toLowerCase();
// 微信环境和安卓微信中可以拨打电话，排除掉在IOS APP中打开
const isSupportTel = ua.match(/MicroMessenger/i) || ua.match(/Android/i) !== null;
class Tel extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>): void {
    const stackComponent = PageStack.getStackComponent();


    if (stackComponent) {
      stackComponent.behavior.trigger<DetailParams['Tel']>({
        actionName: getActionName(sheet.Tel, 0, this.props),
        detailedData: {
          tel: this.props.tel
        }
      });
    }

    if (this.props.onClick) {
      this.props.onClick(e);
    }

    if (isSupportTel) { 
      window.location.href = `tel:${this.props.tel}`;
    }
  }

  public render(): JSX.Element {
    const { children, className, style, id, subscribeContent } = this.props;

    return (
      <Auth
        className={clssNames(className, {
          [styles.disabeld]: !isSupportTel
        })}
        style={style}
        id={id}
        onClick={this.onClick}
        subscribeContent={subscribeContent}
      >{children}</Auth>
    );
  }
}

export default Tel;